/*
 * ParallelPortControl.c: liest aus der Fifo (Pipe) "fifo_pp_command" die Befehle,
 * schreibt ggf. an den Parallel-Port, liesst vom Parallel-Port und
 * schreibt das Ergebnis zurueck in die Fifo "fifo_ppc_answer".
 *
 * Compile with `gcc -O2 -o ParallelPortControl ParallelPortControl.c'
 */

#include <stdio.h>
#include <unistd.h>
#include <asm/io.h>

/* Adresse vom Parallelport: lp1, wenn der nicht passt oder bei meheren PPorts, evtl statt 0x378: 0x278 oder 0x3bc */
/* Bei SUSE in /etc/modules.conf suchen nach "parport"                                                             */
#define BASEPORT 0x378
#define FIFO1 "fifo_pp_command"   /* die Namen der named pipes fr die Uebergabe an das Java-Programm              */
#define FIFO2 "fifo_ppc_answer"   /* FIFO1 fr Kommandos vom Java-Programm, FIFO2 fr die Rckantwort              */

                           /* das Java-Programm sendet nur alle 20 ms ein Kommando !!!                             */
#define PORT_WAIT   1000   /* entspricht  1 ms, diese Zeit wartet dieses Programm zusaetzlich nach Datenausgabe    */

#define TRUE 1
#define FALSE 0
#define DEBUG FALSE        /* viel Info auf Standard-Out  (durch run-Script ins LOG)                               */
#define LOG   TRUE         /* Info ueber Daten an Parallel-Port auf Standard-Out (durch run-Script ins LOG)        */

int main()
{
  char lesestring[80];
  char schreibstring[80];
  char *ptr;
  char a = '0';
  int i, readitem = 0;
  unsigned char data_send = 0;
  unsigned char data_read = 255;
  unsigned char status_read = 0;
  unsigned char control_read = 0;
  unsigned char u;
  int send_command = FALSE;

  FILE *fifo1;
  FILE *fifo2;

  /* Get access to the parallel port */
  if (ioperm(BASEPORT, 3, 1)) {
     perror("ioperm");
     if ( DEBUG ) printf("+++ ParallelPortControl: No Permition for parallel port.\n");
     exit(1);
  }

  /* Set the data signals (D0-7) of the parallel port to all low (0) */
  if ( DEBUG ) printf("+++ ParallelPortControl: Set the data signals (D0-7) of the parallel port to all low (0).\n");
  outb(data_send, BASEPORT);

  /* Sleep for a while (50 ms) */
  usleep(50000);

  /* Check it out ...; wenn das nicht funktioniert, ist etwas nicht in Ordnung. Vielleicht eine falsche BASEPORT-Adresse */
  if ( inb(BASEPORT) != 0 ) {
     printf("+++ ParallelPortControl: Error sending data to parallel port.\n");
     exit(1);
  }

  /* oeffnen der named pipes ... */
  if ( DEBUG ) printf("+++ ParallelPortControl: fifo_pp_command wird geffnet.\n");
  fifo1 = fopen( FIFO1, "r");
  if ( DEBUG ) printf("+++ ParallelPortControl: fifo_pp_command ist geoeffnet.\n");
  if ( DEBUG ) printf("+++ ParallelPortControl: fifo_ppc_answer wird geffnet.\n");
  fifo2 = fopen( FIFO2, "w");
  if ( DEBUG ) printf("+++ ParallelPortControl: fifo_ppc_answer ist geoeffnet.\n");

  /* Wenn das Java-Programm sich beendet, schickt es ein 'q' (quit) als letztes Kommando */
  while ( a != 'q' ) {

    /* was sagt das Java-Programm (alle 20 ms) ... */
    /* Daten am PPort ausgeben: "11000100\n"       */
    /* Nur PPort lesen:         "OnlyRead\n"       */
    /* Programm beenden:        "quit\n"           */
    readitem = fscanf (fifo1, " %s", &lesestring); /* solange fifo nicht geschlossen, wartet das fscanf() */

    if ( DEBUG ) printf("+++ ParallelPortControl <- fifo_pp_command: %s\n", lesestring);
    if ( a == 'q' ) { /* a noch aus der letzten Schleife! Kann eigentlich nicht hier hin gelangen ???? */
       if ( DEBUG ) printf("+++ ParallelPortControl: Set the data signals (D0-7) of the parallel port to all low (0).\n");
       data_send = 0;
       outb(data_send, BASEPORT);
       continue;
    }

    a = *lesestring;               /* Hier wird a das erste Zeichen des Kommando-Strings zugewiesen. */
    if ( readitem > 0 ) {          /* Gibt es neue Kommandos ?                                       */
       if ( a < '0' || a > '1' ) { /* keine Daten am PPort ausgeben                                  */
          if ( DEBUG ) printf("+++ ParallelPortControl: Only read status.\n");
          send_command = FALSE;
       }
       else {                      /* Daten am PPort ausgeben, lesestring -> unsigned char data_send */
          ptr = lesestring;
	  data_send = 0;
	  for (data_send = 0, i=0; i<8; i++, ptr++) {              /* 8 Zeichen werden ausgewertet   */
	     data_send = (data_send<<1) + (*ptr - '0');            /* String verwandeln in data_send */
	  }                                                        /* data_send = 1<<(a-'1');        */

          if ( DEBUG || LOG ) printf("+++ ParallelPortControl: Ausgang %s (%u) wird gesetzt\n", lesestring, data_send );
          send_command = TRUE;
       }

       if ( send_command ) {                                       /* Daten ausgeben                 */   
          /* Set the data signals (D0-7) of the port */
          outb(data_send, BASEPORT);

          /* wait for a while, time for the parallel port */
          usleep(PORT_WAIT);
       }

       /* Read parallel port ... */
       data_read = inb(BASEPORT);
       status_read = inb(BASEPORT + 1);
       control_read = inb(BASEPORT + 2);
       
       /* Data-Bits D0 bis D7: Pin 2 bis 9                                                       */
       /* Status-Bits S3 bis S7 sind nutzbar: Pin 15, 13, 12, 10, 11 (S7=11 invertiert)          */
       /* alle Status-Bits werden invertiert bis auf S7 (mit ^127) , S0 bis S2 auf 0 (mit &0xF8) */
       status_read = (status_read^127)&0xF8;

       /* Control-Bits C0 bis C3 z.Z. nicht genutzt: Pin 1, 14, 16, 17 */

       /* answer to the pipe... */
       ptr = schreibstring;                   /* dieser String soll wieder zurck gegeben werden */
       for (i=0, u=data_read; i<8; i++) {     /* erst die Datenbits (zur Kontrolle) ...          */
          *ptr = '0' + ((u&128)>>7);
	  ptr++;
	  u = u<<1;
       }
       *ptr = ' ';
       ptr++;
       for (i=0, u=status_read; i<8; i++) {   /* ... dann die Statusbits, Info an's Java-Prog.   */
          *ptr = '0' + ((u&128)>>7);
	  ptr++;
	  u = u<<1;
       }
       *ptr = '\0';

       /* Jetzt sieht schreibstring z.B. so aus: "11000100 00001000\0", erst Data, dann Status   */

       if ( DEBUG ) printf("+++ ParallelPortControl: %s: Data = %u; Status = %u; Control = %u.\n",
							schreibstring,  data_read,  status_read,  control_read);

       fprintf(fifo2, "%s\n", schreibstring); /* Antwort an das Java-Programm ueber die fifo     */
       fflush (fifo2);
       if ( DEBUG ) printf("+++ ParallelPortControl -> fifo_ppc_answer: %s.\n", schreibstring);
    }
    else {        /* keine Kommandos vom Java-Programm! */
        a = 'q';  /* dann beendet sich dieses Programm. */
    }
  }

  fclose(fifo1);
  fclose(fifo2);

  /* We don't need the parallel port anymore */
  if (ioperm(BASEPORT, 3, 0)) {
     perror("ioperm");
     exit(1);
  }

  exit(0);
}





