/*
 * ParallelPortTest.c: Testet den Zugriff auf den Parallel-Port.
 * Nach einem fork liest der eine Prozess alle 50 ms die Status-Pins
 * und gibt Aenderungen bekannt.
 * Der Vaterprozess wartet auf Eingaben auf StdIn und schreibt diese
 * an die Data-Pins des Parallel-Ports. Danach liesst er diese Werte
 * zum Vergleich wieder vom Parallel-Port ein.
 *
 * Compile with `gcc -O2 -o ParallelPortTest ParallelPortTest.c'
 */

#include <stdio.h>
#include <unistd.h>
#include <asm/io.h>
#include <sys/types.h>

/* Adresse vom Parallelport: lp1, wenn der nicht passt oder bei meheren PPorts, evtl statt 0x378: 0x278 oder 0x3bc */
/* Bei SUSE in /etc/modules.conf suchen nach "parport"                                                             */
#define BASEPORT 0x378

#define TRUE 1
#define FALSE 0

int main (int argc, char *argv[])
{
  char lesestring[80];
  char statusControl[80];
  char statusControlNeu[80];
  char *ptr;
  char a = '0';
  int i, readitem = 0;
  int id;         /* id = 0 fuer Vaterprozess; id = Prozessnummer fuer Sohnprozess */
  unsigned char data_send = 0;
  unsigned char data_read = 255;
  unsigned char status_read = 0;
  unsigned char control_read = 0;
  unsigned char u;

  if (argc != 1 )
  {
     printf("usage: %s \n", argv[0]);
     exit (-1);
  }

  /* Get access to the parallel port */
  if (ioperm(BASEPORT, 3, 1)) {
     perror("ioperm");
     printf("+++ ParallelPortTest: No Permition for parallel port. Change to root!\n");
     exit(1);
  }

  /* Set the data signals (D0-7) of the parallel port to all low (0) */
  printf("+++ ParallelPortControl: Set the data signals (D0-7) of the parallel port to all low (0).\n");
  outb(data_send, BASEPORT);

  /* Sleep for a while (50 ms) */
  usleep(50000);

  /* Check it out ...; wenn das nicht funktioniert, ist etwas nicht in Ordnung. Vielleicht eine falsche BASEPORT-Adresse */
  if ( inb(BASEPORT) != 0 ) {
     printf("+++ ParallelPortControl: Error sending data to parallel port.\n");
     exit(1);
  }

  id = fork();

  if( id < 0 )
  {
     printf("Beim fork ist ein Fehler aufgetreten!");
     exit(-1);
  }

  if ( id == 0 ) /* Vaterprozess liest Standard-Eingabe */
  {
     printf("Bitte Data-Werte eintippen (z.B.11000100) oder 'Contl C' fr Ende: \n");
     while ( a != 'q' )
     {
       strcpy(lesestring, "00000000");
       readitem = scanf ("%s", &lesestring); /* solange fifo nicht geschlossen, wartet das fscanf() */
       if ( *lesestring != '0' && *lesestring != '1' ) a = 'q';
       ptr = lesestring;
       for (i=0; i<8; i++)
       {
	  if (*ptr != '1') *ptr = '0';
	  ptr++;
       }
       *ptr = '\0';
       ptr = lesestring;
       data_send = 0;
       for (data_send = 0, i=0; i<8; i++, ptr++) {                /* 8 Zeichen werden ausgewertet   */
	    data_send = (data_send<<1) + (*ptr - '0');            /* String verwandeln in data_send */
       }                                                          /* data_send = 1<<(a-'1');        */

       printf("Data-Werte werden gesetzt: %s (%u) \n", lesestring, data_send );
       /* Set the data signals (D0-7) of the port */
       outb(data_send, BASEPORT);

       /* wait for a while (10 ms) */
       usleep(10000);

       /* Kontrolle ... */
       data_read = inb(BASEPORT);
       if ( data_read != data_send )
          printf("Data-Werte konnten nicht korrekt gesetzt werden: %u (%u) \n", data_read, data_send );
     }
  }
  if ( id > 0 )  /* Sohn Prozess liest Status und Control-Werte vom PP */
  {
     *statusControl = '\0';
     while ( TRUE )
     {
       status_read = inb(BASEPORT + 1);
       /* Data-Bits D0 bis D7: Pin 2 bis 9                                                       */
       /* Status-Bits S3 bis S7 sind nutzbar: Pin 15, 13, 12, 10, 11 (S7=11 invertiert)          */
       /* alle Status-Bits werden invertiert bis auf S7 (mit ^127) , S0 bis S2 auf 0 (mit &0xF8) */
       status_read = (status_read^127)&0xF8;
       ptr = statusControlNeu;
       for (i=0, u=status_read; i<8; i++) {   /* ... dann die Statusbits, Info an's Java-Prog.   */
          *ptr = '0' + ((u&128)>>7);
	  ptr++;
	  u = u<<1;
       }
       *ptr = '\0';
       if ( strcmp(statusControlNeu, statusControl) ) {
	       printf("neue Werte an Status-Pins: %s \n", statusControlNeu);
	       strcpy (statusControl, statusControlNeu);
       }
       /* Sleep for a while (50 ms) */
       usleep(50000);
     }
  }

  /* We don't need the parallel port anymore */
  if (ioperm(BASEPORT, 3, 0)) {
     perror("ioperm");
     exit(1);
  }

  exit(0);
}





