#!/bin/sh

# Startscript fr die SchiriAnlage
# Argumente: keine
#
# Author: Ulrich Wller
# Version 1.0 21.05.2003

NR_ARGS=0  # Argumente ohne den Aufrufnamen

GET_OUT_OF_HERE()
{
        # Zum Schluss noch aufraemen ...
	rm -f fifo_pp_command fifo_ppc_answer > /dev/null
        # und Ende ...
        exit $1
}

trap 'echo "... abgebrochen mit SIGHUP ..."; GET_OUT_OF_HERE 3' 1  # kill process
trap 'echo "... abgebrochen mit SIGINT ..."; GET_OUT_OF_HERE 3' 2  # Control C
trap 'echo "... abgebrochen mit SIGQUIT ..."; GET_OUT_OF_HERE 3' 3
trap 'echo "... abgebrochen mit SIGKILL ..."; GET_OUT_OF_HERE 3' 9 # cannot catch!
trap 'echo "... abgebrochen mit SIGTERM ..."; GET_OUT_OF_HERE 3' 15

USAGE()
{
	echo "Startscript fr die SchiriAnlage. Aufruf ohne Argumente."
        echo "usage: $0"
}

if [ $# -ne "$NR_ARGS" ]
then
        USAGE
        GET_OUT_OF_HERE
fi

# Only one instance of this Script should run !!!
SCRIPTNAME=`basename $0`
PID=$$

ps axl | fgrep $SCRIPTNAME | fgrep -v fgrep | fgrep -v $PID >/dev/null
if [ $? -eq 0 ] # another instance is running ...
then
	xmessage "Das Programm kann nicht gleichzeitig mehrfach laufen (Doppelklick?)!"
	exit 255
fi

# change directory to directory of start script ...
cd `dirname $0`

# check java runtime environment ...
type java > /dev/null
JAVAOK=$?
if [ $JAVAOK -ne 0 ]
then
	echo "SORRY, NO JAVA RUNTIME ENVIRONMENT AVAILABLE! INSTALL java!"
	exit -1
fi

# check java compiler ...
type javac > /dev/null
JAVACOK=$?

# check all class files ...
for classfile in ParallelPortKommunikation.class SchiriAnlage.class Strafzeit.class uwrUhr.class
do
	if [ ! -f $classfile ]
	then
		JAVACOMPILE=TRUE
	fi
done

# check for newer java file ...
if [ \( -f SchiriAnlage.java \) -a \(  -f SchiriAnlage.class \) -a \
     \( SchiriAnlage.java -nt SchiriAnlage.class \) ]
then
	if [ $JAVACOK -eq 0 ]
	then
		JAVACOMPILE=TRUE
	else
		if [ "$JAVACOMPILE" = "TRUE" ]
		then
			echo "SORRY, MUST COMPILE SchiriAnlage.java! BUT NO JAVA COMPILER AVAILABLE!"
			exit -1
		else
			echo "Ignoring the newer SchiriAnlage.java. No Java compiler available."
		fi
	fi
fi

if [ "$JAVACOMPILE" = "TRUE" ]
then
	if [ ! -f SchiriAnlage.java ]
	then
		echo "SORRY, MUST COMPILE SchiriAnlage.java! BUT NO SOURCE FILE AVAILABLE!"
		exit -1
	fi
	if [ $JAVACOK -eq 0 ]
	then
		echo "SchiriAnlage.java will be compiled."
		javac SchiriAnlage.java
		if [ $? -ne 0 ]
		then
			echo 'SORRY, ERROR OCCURED! CHECK JAVA VERSION ( > 1.1 ) AND SOURCE FILE!'
			java -version
			ls -l SchiriAnlage.java
			exit -1
		fi
	else
		echo "SORRY, NO JAVA COMPILER AVAILABLE! INSTALL jdk!"
	fi
fi

# check compiler gcc ...
type gcc > /dev/null
GCCOK=$?

# check whether ParallelPortControl exists and is executable ...
if [ ! -x ParallelPortControl ]
then
	echo "ParallelPortControl must be compiled ..."
	if [ $GCCOK -eq 0 ]
	then
		if [ -f ParallelPortControl.c ]
		then
			gcc -O2 -o ParallelPortControl ParallelPortControl.c
		else
			echo "SORRY, NO SOURCE FILE ParallelPortControl.c AVAILABLE!"
			exit -1
		fi
	else
		echo "SORRY, NO COMPILER AVAILABLE! INSTALL gcc!"
		exit -1
	fi
fi

# check whether ParallelPortControl.c is newer ...
if [ \( -f ParallelPortControl.c \) -a \( ParallelPortControl.c -nt ParallelPortControl \) ]
then
	echo "ParallelPortControl (too old) should be compiled ..."
	if [ $GCCOK -eq 0 ]
	then
		gcc -O2 -o ParallelPortControl ParallelPortControl.c
	else
		echo "No compiler available. The old version of ParallelPortControl is used."
	fi
fi

# check whether ParallelPortControl belongs root with s-bit set ...
ls -l ParallelPortControl | fgrep root | fgrep 'rwsr-sr-x' > /dev/null
RechteOK=$?
while [ $RechteOK -ne 0 ]
do
	echo "ParallelPortControl an root mit s-Bit. Bitte root-Password eingeben."
	su -c 'chown root ParallelPortControl; chmod 6755 ParallelPortControl'
	ls -l ParallelPortControl | fgrep root | fgrep 'rwsr-sr-x' > /dev/null
	RechteOK=$?
done

# check fifos ...
rm -f fifo_pp_command fifo_ppc_answer > /dev/null
mkfifo fifo_pp_command fifo_ppc_answer
if [ ! -p fifo_pp_command ]
then
	echo "SORRY, NAMED PIPE (FIFO) fifo_pp_command CANNOT BE CREATED! CHECK IT!"
	exit -1
fi
if [ ! -p fifo_ppc_answer ]
then
        echo "SORRY, NAMED PIPE (FIFO) fifo_ppc_answer CANNOT BE CREATED! CHECK IT!"
        exit -1
fi

echo "All checks passed!"

######################################################################################
# run it now ...

# run ParallelPortControl in background ...
./ParallelPortControl > ParallelPortControl.log &

# run the Java-Programm ...
java SchiriAnlage

GET_OUT_OF_HERE 0


